#! /usr/bin/env python3
from flask import Flask, render_template, request, redirect, flash
from schmeckels.helper import get_session, format_amount
from datetime import datetime, timedelta, date
from sqlalchemy import func, and_
import click
import calendar
from schmeckels.models import Transaction, Tag
from schmeckels.helper import import_transactions
from schmeckels import banks

session = None
app = Flask(__name__)


@app.route("/")
def index():
    tag_sum_in = {}
    tag_sum_out = {}

    # Get a list of all tags
    tags = session.query(Tag).all()

    # Get start and end of last month
    now = datetime.now()
    last_month = now - timedelta(days=now.day)
    num_days = calendar.monthrange(last_month.year, 2)[1]
    start_date = date(last_month.year, 2, 1)
    end_date = date(last_month.year, 2, num_days)

    for tag in tags:
        tag_sum = (
            session.query(func.sum(Transaction.amount))
            .filter(and_(Transaction.date >= start_date, Transaction.date <= end_date))
            .filter(Transaction.tags.any(id=tag.id))
            .first()[0]
        )
        if tag_sum:
            if tag_sum >= 0:
                tag_sum_in[tag] = tag_sum
            else:
                tag_sum_out[tag] = tag_sum

    tag_sum_in = {
        k: format_amount(v) for k, v in sorted(tag_sum_in.items(), key=lambda item: float(item[1]), reverse=True)
    }
    tag_sum_out = {k: format_amount(v) for k, v in sorted(tag_sum_out.items(), key=lambda item: float(item[1]))}
    return render_template("index.html", tag_sum_in=tag_sum_in, tag_sum_out=tag_sum_out, date=start_date)


@app.route("/tags")
def tags():
    tags = session.query(Tag).all()
    return render_template("tags.html", tags=tags)


@app.route("/tag/<name>")
def tag(name):
    tag = session.query(Tag).filter(Tag.name == name).first()
    transactions = session.query(Transaction).filter(Transaction.tags.any(id=tag.id)).all()

    return render_template("tag.html", tag=tag, transactions=transactions)


@app.route("/tag/<name>/delete")
def delete_tag(name):
    x = session.query(Tag).filter(Tag.name == name).first()
    if x:
        session.delete(x)
        session.commit()

    return redirect("/tags")


@app.route("/tag/<name>/toggle")
def toggle_tag(name):
    x = session.query(Tag).filter(Tag.name == name).first()
    if x:
        x.toggle_reporting()
        session.add(x)
        session.commit()

    return redirect("/tags")


@app.route("/transactions")
def transactions():
    transactions = session.query(Transaction)

    if request.args.get("unsorted", None):
        transactions = transactions.filter(Transaction.tags == None)

    transactions = transactions.all()

    tags = session.query(Tag).all()
    return render_template("transactions.html", transactions=transactions, tags=tags)


@app.route("/bulksort", methods=["POST"])
def buksort():
    tag_id = request.form.get("tag")
    tag = session.query(Tag).get(tag_id)
    transaction_ids = request.form.getlist("transaction")

    for id in transaction_ids:
        t = session.query(Transaction).get(id)
        t.tags.append(tag)
        session.add(t)
    session.commit()

    return redirect("/transactions")


@app.route("/upload", methods=["GET", "POST"])
def upload():
    global session

    if request.method == "POST":
        file = request.files.get("file")
        if not file or file.filename == "":
            flash("No file choosen")
            return redirect(request.url)

        path = f"/tmp/{file.filename}"
        file.save(path)
        new = import_transactions(session, request.form["bank"], path)
        if len(new) > 0:
            session.bulk_save_objects(new)
            session.commit()
        flash(f"Imported {len(new)} new transactions")
        return render_template("upload.html", banks=banks.SUPPORTED_BANKS, transactions=new)

    return render_template("upload.html", banks=banks.SUPPORTED_BANKS)


@click.command(name="serve")
@click.option("--profile", "-p")
def command(profile):
    global session
    session = get_session(profile)
    app.secret_key = "WeDon'tHaveSessionsSoThisMustNotBeVerySecure!"
    app.run(host="0.0.0.0", port=8080)
